==> lmtpd-0.9.19.tar.gz: lmtpd-0.9.19/spam-assassin.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

static char const rcsid[] =
  "$Id: spam-assassin.c,v 1.12 2004/01/30 15:02:33 lwa Exp $";

#include "config.h"

#include <sys/types.h>
#include <sys/socket.h>

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>

#include "lmtp.h"

#define ASSASSIN_VERSION "SPAMC/1.2"

#define ASSASSIN_BUFLEN 8192
#define SPAM_LEVEL_CHAR '*'

void spam_assassin_init(struct fstate *state) {
  rxinit(&state->spamassassin, 0, NULL);
}

int spam_assassin(struct fstate *state, char *prefix) {
  char buf[ASSASSIN_BUFLEN];
  int len;
  ssize_t maillen;
  int fd;
  float version;
  int response;
  char *rbody;
  char *acl_ret;
  if (state->spamassassin_maxsize > 0 &&
      state->msgsize > state->spamassassin_maxsize)
    return 0;

  if (state->spamassassin_done)
    return 0;

  acl_ret = acl_mailbox(&(state->spamassassin_acl), state->rcpt->mbox);

  if (acl_ret != NULL && *acl_ret != 0)
    return 0;

  state->spamassassin_done = 1;

  fd = rxconnect(&state->spamassassin);
  if (fd == -1)
    return -1;
  maillen = mailwrite_size(state, state->hdr_delivered, HEADER_OTHER);
  len = snprintf(buf, sizeof(buf),
		 "SYMBOLS " ASSASSIN_VERSION "\r\n"
		 "User: %s\r\n"
		 "Content-length: %ld\r\n"
		 "\r\n",
		 state->rcpt->mbox,
		 (long)maillen
		 );
  if (write(fd, buf, len) == -1 ||
      mailwrite(state, state->hdr_delivered, HEADER_OTHER, fd) == -1) {
    roll_server(&state->spamassassin, "SpamAssassin server %s: %s",
		current_server(&state->spamassassin), strerror(errno));
  }
  shutdown(fd, SHUT_WR);
  len = 0;
  while(len < sizeof(buf)) {
    int i = read(fd, buf + len, sizeof(buf)-len);
    if (i == 0)
      break;
    if (i < 0) {
      if (len == 0)
	roll_server(&state->spamassassin, "SpamAssassin server %s: %s",
		    current_server(&state->spamassassin), strerror(errno));
      break; /* read error */
    }
    len += i;
  }
  rxclose(&state->spamassassin);
  buf[len] = 0;
  rbody = memchr(buf, '\n', len);
  if (rbody == NULL) 
    return -1;
  *rbody++ = 0;
  if (sscanf(buf, "SPAMD/%f %d %*s", &version, &response) != 2) {
    roll_server(&state->spamassassin, 
		"SpamAssassin server %s use unknown protocol",
		current_server(&state->spamassassin));
    return -1;
  }
  if (response != 0) { /* ! EX_OK */
    roll_server(&state->spamassassin, "SpamAssassin server %s returns %d\n",
		current_server(&state->spamassassin), response);
    return -1;
  }

  if (version - 1.0 > 0.01) {
    int len;
    char *symbols;
    struct headers *newh;
    double score;
    double threshold;
    char spam_truefalse[5];
    int is_spam;
    char tests[ASSASSIN_BUFLEN] = "";

    if (sscanf(rbody, "Spam: %5s ; %lf / %lf%n",
	       spam_truefalse, &score, &threshold, &len) != 3) {
      roll_server(&state->spamassassin,
		  "SpamAssassin server %s use unknown protocol\n",
		  current_server(&state->spamassassin));
      return -1;
    }
    rbody += len;
    while(isspace((int)*rbody)) rbody++;
    symbols = rbody;
    while(isprint((int)*symbols)) symbols++;
    *symbols = 0;

    is_spam = (strcasecmp(spam_truefalse, "true") == 0);


    /* split result in lines */
    /* lot of work for pretty result */
#define TESTS_CUT 78
#define TESTS_TAB    "              "
#define TESTS_INTRO  "        tests="     

    if (*rbody) {
      int tests_len = 0;
      memcpy(tests, TESTS_INTRO, sizeof(TESTS_INTRO)-1);
      tests_len += sizeof(TESTS_INTRO) - 1;
    
      while(tests_len < sizeof(tests)) {
	int cut = 0;
	int pos;
	for (pos = 0;
	     pos < (TESTS_CUT - (int)sizeof(TESTS_TAB));
	     pos ++) {
	  if (rbody[pos] == ',' || rbody[pos] == 0) cut = pos;
	}
	if (cut == 0) {
	  while(rbody[pos] && rbody[pos] != ',') pos++;
	  cut = pos;
	}
	if (rbody[cut] == ',')
	  cut++;
	memcpy(tests + tests_len, rbody, cut);
	tests_len += cut;
	tests[tests_len++] = '\n';
	rbody += cut;
	if (*rbody) {
	  memcpy(tests + tests_len, TESTS_TAB, sizeof(TESTS_TAB)-1);
	  tests_len += sizeof(TESTS_TAB)-1;
	} else {
	  while(tests[tests_len-1] == '\n') tests[--tests_len] = 0;
	  break;
	}
      }
    }

    if (prefix) { /* purify prefix */
      char *c = prefix;
      while(*c && *c >= 33 && *c <= 126 && *c != ':') /* RFC 2822; 2.2 */
	c++;
      *c = 0;
      if (*prefix == 0)
	prefix = "X-Spam";
    } else {
      prefix = "X-Spam";
    }

    if ((newh = setheader(state->hlevel, "%s-Status: %s, "
			  "hits=%.1f required=%.1f\n%s",
			  prefix,
			  (is_spam ? "Yes" : "No"),
			  score, threshold, tests)) == NULL)
      return -1;

    newh->trace |= TRACE_ON;
    newh->next = state->hdr_delivered->next;
    state->hdr_delivered->next = newh;
    state->nbheaders++;
    state->spamassassin_score = score;
    {
      char spamlevel[100];
      unsigned i = 0;
      while(i <= score-1 && i < sizeof(spamlevel)) {
	spamlevel[i++] = SPAM_LEVEL_CHAR;
      }
      spamlevel[i] = 0;
      if ((newh = setheader(state->hlevel, "%s-Level: %s", 
			    prefix, spamlevel))
	  != NULL) {
	newh->trace |= TRACE_ON;
	newh->next = state->hdr_delivered->next;
	state->hdr_delivered->next = newh;
	state->nbheaders++;
      }
    }
    return 0;

  } else {
      roll_server(&state->spamassassin,
		  "SpamAssassin server %s use obsolete version %.2f\n",
		  current_server(&state->spamassassin), version);
    return -1;
  }
}

==> lmtpd-0.9.24.tar.gz: lmtpd-0.9.24/spam-assassin.c <==
/*
   Copyright 2000, 2001, 2002, 2003 Laurent Wacrenier

   This file is part of PLL lmtpd.

   PLL lmtpd is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   PLL lmtpd is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with PLL lmtpd; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

*/

#include "config.h"

static char const rcsid[] UNUSED =
  "$Id: spam-assassin.c,v 1.13 2004/10/26 09:59:20 lwa Exp $";



#include <sys/types.h>
#include <sys/socket.h>

#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <ctype.h>

#include "lmtp.h"

#define ASSASSIN_VERSION "SPAMC/1.2"

#define ASSASSIN_BUFLEN 8192
#define SPAM_LEVEL_CHAR '*'

void spam_assassin_init(struct fstate *state) {
  rxinit(&state->spamassassin, 0, NULL);
}

int spam_assassin(struct fstate *state, char *prefix) {
  char buf[ASSASSIN_BUFLEN];
  int len;
  ssize_t maillen;
  int fd;
  float version;
  int response;
  char *rbody;
  char *acl_ret;
  if (state->spamassassin_maxsize > 0 &&
      state->msgsize > state->spamassassin_maxsize)
    return 0;

  if (state->spamassassin_done)
    return 0;

  acl_ret = acl_mailbox(&(state->spamassassin_acl), state->rcpt->mbox);

  if (acl_ret != NULL && *acl_ret != 0)
    return 0;

  state->spamassassin_done = 1;

  fd = rxconnect(&state->spamassassin);
  if (fd == -1)
    return -1;
  maillen = mailwrite_size(state, state->hdr_delivered, HEADER_OTHER);
  len = snprintf(buf, sizeof(buf),
		 "SYMBOLS " ASSASSIN_VERSION "\r\n"
		 "User: %s\r\n"
		 "Content-length: %ld\r\n"
		 "\r\n",
		 state->rcpt->mbox,
		 (long)maillen
		 );
  if (write(fd, buf, len) == -1 ||
      mailwrite(state, state->hdr_delivered, HEADER_OTHER, fd) == -1) {
    roll_server(&state->spamassassin, "SpamAssassin server %s: %s",
		current_server(&state->spamassassin), strerror(errno));
  }
  shutdown(fd, SHUT_WR);
  len = 0;
  while(len < sizeof(buf)) {
    int i = read(fd, buf + len, sizeof(buf)-len);
    if (i == 0)
      break;
    if (i < 0) {
      if (len == 0)
	roll_server(&state->spamassassin, "SpamAssassin server %s: %s",
		    current_server(&state->spamassassin), strerror(errno));
      break; /* read error */
    }
    len += i;
  }
  rxclose(&state->spamassassin);
  buf[len] = 0;
  rbody = memchr(buf, '\n', len);
  if (rbody == NULL) 
    return -1;
  *rbody++ = 0;
  if (sscanf(buf, "SPAMD/%f %d %*s", &version, &response) != 2) {
    roll_server(&state->spamassassin, 
		"SpamAssassin server %s use unknown protocol",
		current_server(&state->spamassassin));
    return -1;
  }
  if (response != 0) { /* ! EX_OK */
    roll_server(&state->spamassassin, "SpamAssassin server %s returns %d\n",
		current_server(&state->spamassassin), response);
    return -1;
  }

  if (version - 1.0 > 0.01) {
    int len;
    char *symbols;
    struct headers *newh;
    double score;
    double threshold;
    char spam_truefalse[5];
    int is_spam;
    char tests[ASSASSIN_BUFLEN] = "";

    if (sscanf(rbody, "Spam: %5s ; %lf / %lf%n",
	       spam_truefalse, &score, &threshold, &len) != 3) {
      roll_server(&state->spamassassin,
		  "SpamAssassin server %s use unknown protocol\n",
		  current_server(&state->spamassassin));
      return -1;
    }
    rbody += len;
    while(isspace((int)*rbody)) rbody++;
    symbols = rbody;
    while(isprint((int)*symbols)) symbols++;
    *symbols = 0;

    is_spam = (strcasecmp(spam_truefalse, "true") == 0);


    /* split result in lines */
    /* lot of work for pretty result */
#define TESTS_CUT 78
#define TESTS_TAB    "              "
#define TESTS_INTRO  "        tests="     

    if (*rbody) {
      int tests_len = 0;
      memcpy(tests, TESTS_INTRO, sizeof(TESTS_INTRO)-1);
      tests_len += sizeof(TESTS_INTRO) - 1;
    
      while(tests_len < sizeof(tests)) {
	int cut = 0;
	int pos;
	for (pos = 0;
	     pos < (TESTS_CUT - (int)sizeof(TESTS_TAB));
	     pos ++) {
	  if (rbody[pos] == ',' || rbody[pos] == 0) cut = pos;
	}
	if (cut == 0) {
	  while(rbody[pos] && rbody[pos] != ',') pos++;
	  cut = pos;
	}
	if (rbody[cut] == ',')
	  cut++;
	memcpy(tests + tests_len, rbody, cut);
	tests_len += cut;
	tests[tests_len++] = '\n';
	rbody += cut;
	if (*rbody) {
	  memcpy(tests + tests_len, TESTS_TAB, sizeof(TESTS_TAB)-1);
	  tests_len += sizeof(TESTS_TAB)-1;
	} else {
	  while(tests[tests_len-1] == '\n') tests[--tests_len] = 0;
	  break;
	}
      }
    }

    if (prefix) { /* purify prefix */
      char *c = prefix;
      while(*c && *c >= 33 && *c <= 126 && *c != ':') /* RFC 2822; 2.2 */
	c++;
      *c = 0;
      if (*prefix == 0)
	prefix = "X-Spam";
    } else {
      prefix = "X-Spam";
    }

    if ((newh = setheader(state->hlevel, "%s-Status: %s, "
			  "hits=%.1f required=%.1f\n%s",
			  prefix,
			  (is_spam ? "Yes" : "No"),
			  score, threshold, tests)) == NULL)
      return -1;

    newh->trace |= TRACE_ON;
    newh->next = state->hdr_delivered->next;
    state->hdr_delivered->next = newh;
    state->nbheaders++;
    state->spamassassin_score = score;
    {
      char spamlevel[100];
      unsigned i = 0;
      while(i <= score-1 && i < sizeof(spamlevel)) {
	spamlevel[i++] = SPAM_LEVEL_CHAR;
      }
      spamlevel[i] = 0;
      if ((newh = setheader(state->hlevel, "%s-Level: %s", 
			    prefix, spamlevel))
	  != NULL) {
	newh->trace |= TRACE_ON;
	newh->next = state->hdr_delivered->next;
	state->hdr_delivered->next = newh;
	state->nbheaders++;
      }
    }
    return 0;

  } else {
      roll_server(&state->spamassassin,
		  "SpamAssassin server %s use obsolete version %.2f\n",
		  current_server(&state->spamassassin), version);
    return -1;
  }
}
